@@ -72,7 +72,7 @@ struct archive_write {
 
 	/* Dev/ino of the archive being written. */
 	int		  skip_file_set;
-	dev_t		  skip_file_dev;
+	int64_t		  skip_file_dev;
 	int64_t		  skip_file_ino;
 
 	/* Utility:  Pointer to a block of nulls. */
