@@ -1,6 +1,6 @@
 /*-
- * Copyright (c) 2009,2011 Michihiro NAKAJIMA
  * Copyright (c) 2008 Joerg Sonnenberger
+ * Copyright (c) 2009-2012 Michihiro NAKAJIMA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -389,13 +389,13 @@ dir_len(struct mtree_entry *me)
 		else if (*p == '/')
 			al = p - path;
 	}
-	if (l == -1)
+	if (l == (size_t)-1)
 		goto alen;
 	size = p - path;
 	rp = p = path;
 	while (*p != '\0') {
 		l = mbtowc(&wc, p, size);
-		if (l == -1)
+		if (l == (size_t)-1)
 			goto alen;
 		if (l == 1 && (wc == L'/' || wc == L'\\'))
 			rp = p;
@@ -404,7 +404,7 @@ dir_len(struct mtree_entry *me)
 	}
 	return (rp - path + 1);
 alen:
-	if (al == -1)
+	if (al == (size_t)-1)
 		return (0);
 	return (al + 1);
 }
@@ -553,7 +553,8 @@ write_global(struct mtree_writer *mtree)
 	}
 	if ((keys & effkeys & F_MODE) != 0) {
 		mtree->set.mode = mtree->set.mode_list->m_entry->mode;
-		archive_string_sprintf(&setstr, " mode=%o", mtree->set.mode);
+		archive_string_sprintf(&setstr, " mode=%o",
+		    (unsigned int)mtree->set.mode);
 	}
 	if ((keys & effkeys & F_FLAGS) != 0) {
 		name = mtree->set.flags_list->m_entry->fflags_text;
@@ -913,7 +914,7 @@ write_entry(struct archive_write *a, struct mtree_entry *me)
 		archive_string_sprintf(str, " time=%jd.%jd",
 		    (intmax_t)me->mtime, (intmax_t)me->mtime_nsec);
 	if ((keys & F_MODE) != 0)
-		archive_string_sprintf(str, " mode=%o", me->mode);
+		archive_string_sprintf(str, " mode=%o", (unsigned int)me->mode);
 	if ((keys & F_GID) != 0)
 		archive_string_sprintf(str, " gid=%jd", (intmax_t)me->gid);
 	if ((keys & F_UID) != 0)
@@ -1078,7 +1079,7 @@ archive_write_mtree_data(struct archive_write *a, const void *buff, size_t n)
 	struct mtree_writer *mtree= a->format_data;
 
 	if (n > mtree->entry_bytes_remaining)
-		n = mtree->entry_bytes_remaining;
+		n = (size_t)mtree->entry_bytes_remaining;
 	mtree->entry_bytes_remaining -= n;
 
 	/* We don't need to compute a regular file sum */
@@ -1221,7 +1222,10 @@ archive_write_mtree_options(struct archive_write *a, const char *key,
 		return (ARCHIVE_OK);
 	}
 
-	return (ARCHIVE_FAILED);
+	/* Note: The "warn" return is just to inform the options
+	 * supervisor that we didn't handle it.  It will generate
+	 * a suitable error if no one used this option. */
+	return (ARCHIVE_WARN);
 }
 
 int
