@@ -10,6 +10,11 @@ DEFINE_TEST(test_archive_sha1)
 DEFINE_TEST(test_archive_sha256)
 DEFINE_TEST(test_archive_sha384)
 DEFINE_TEST(test_archive_sha512)
+DEFINE_TEST(test_archive_getdate)
+DEFINE_TEST(test_archive_match_owner)
+DEFINE_TEST(test_archive_match_path)
+DEFINE_TEST(test_archive_match_time)
+DEFINE_TEST(test_archive_pathmatch)
 DEFINE_TEST(test_archive_read_close_twice)
 DEFINE_TEST(test_archive_read_close_twice_open_fd)
 DEFINE_TEST(test_archive_read_close_twice_open_filename)
