@@ -23,10 +23,8 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include "test.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/test/test_write_format_cpio.c,v 1.4 2008/01/01 22:28:04 kientzle Exp $");
+__FBSDID("$FreeBSD: head/lib/libarchive/test/test_write_format_cpio.c 185672 2008-12-06 06:02:26Z kientzle $");
 
-/* The version stamp macro was introduced after cpio write support. */
-#if ARCHIVE_VERSION_STAMP >= 1009000
 static void
 test_format(int	(*set_format)(struct archive *))
 {
@@ -87,6 +85,48 @@ test_format(int	(*set_format)(struct archive *))
 	archive_entry_free(ae);
 	assertA(4 == archive_write_data(a, "1234", 5));
 
+	/*
+	 * Write a file with a name, filetype, and size.
+	 */
+	assert((ae = archive_entry_new()) != NULL);
+	archive_entry_copy_pathname(ae, "name");
+	archive_entry_set_size(ae, 0);
+	archive_entry_set_filetype(ae, AE_IFREG);
+	assertEqualInt(ARCHIVE_OK, archive_write_header(a, ae));
+	assert(archive_error_string(a) == NULL);
+	archive_entry_free(ae);
+
+	/*
+	 * Write a file with a name and filetype but no size.
+	 */
+	assert((ae = archive_entry_new()) != NULL);
+	archive_entry_copy_pathname(ae, "name");
+	archive_entry_unset_size(ae);
+	archive_entry_set_filetype(ae, AE_IFREG);
+	assertEqualInt(ARCHIVE_FAILED, archive_write_header(a, ae));
+	assert(archive_error_string(a) != NULL);
+	archive_entry_free(ae);
+
+	/*
+	 * Write a file with a name and size but no filetype.
+	 */
+	assert((ae = archive_entry_new()) != NULL);
+	archive_entry_copy_pathname(ae, "name");
+	archive_entry_set_size(ae, 0);
+	assertEqualInt(ARCHIVE_FAILED, archive_write_header(a, ae));
+	assert(archive_error_string(a) != NULL);
+	archive_entry_free(ae);
+
+	/*
+	 * Write a file with a size and filetype but no name.
+	 */
+	assert((ae = archive_entry_new()) != NULL);
+	archive_entry_set_size(ae, 0);
+	archive_entry_set_filetype(ae, AE_IFREG);
+	assertEqualInt(ARCHIVE_FAILED, archive_write_header(a, ae));
+	assert(archive_error_string(a) != NULL);
+	archive_entry_free(ae);
+
 	/*
 	 * Write a directory to it.
 	 */
@@ -103,15 +143,13 @@ test_format(int	(*set_format)(struct archive *))
 
 
 	/* Close out the archive. */
-	assertA(0 == archive_write_close(a));
-#if ARCHIVE_API_VERSION > 1
-	assertA(0 == archive_write_finish(a));
-#else
-	archive_write_finish(a);
-#endif
+	assertEqualIntA(a, ARCHIVE_OK, archive_write_close(a));
+	assertEqualInt(ARCHIVE_OK, archive_write_free(a));
 
 	/*
 	 * Damage the second entry to test the search-ahead recovery.
+	 * TODO: Move the damage-recovery checking to a separate test;
+	 * it doesn't really belong in this write test.
 	 */
 	{
 		int i;
@@ -124,17 +162,20 @@ test_format(int	(*set_format)(struct archive *))
 		}
 	}
 	failure("Unable to locate the second header for damage-recovery test.");
-	assert(damaged = 1);
+	assert(damaged == 1);
 
 	/*
 	 * Now, read the data back.
 	 */
 	assert((a = archive_read_new()) != NULL);
 	assertA(0 == archive_read_support_format_all(a));
-	assertA(0 == archive_read_support_compression_all(a));
+	assertA(0 == archive_read_support_filter_all(a));
 	assertA(0 == archive_read_open_memory(a, buff, used));
 
-	assertEqualIntA(a, 0, archive_read_next_header(a, &ae));
+	if (!assertEqualIntA(a, 0, archive_read_next_header(a, &ae))) {
+		archive_read_free(a);
+		return;
+	}
 
 	assertEqualInt(1, archive_entry_mtime(ae));
 	/* Not the same as above: cpio doesn't store hi-res times. */
@@ -145,31 +186,23 @@ test_format(int	(*set_format)(struct archive *))
 	assertEqualInt((S_IFREG | 0755), archive_entry_mode(ae));
 	assertEqualInt(8, archive_entry_size(ae));
 	assertA(8 == archive_read_data(a, filedata, 10));
-	assert(0 == memcmp(filedata, "12345678", 8));
+	assertEqualMem(filedata, "12345678", 8);
 
 	/*
-	 * Read the second file back.
+	 * The second file can't be read because we damaged its header.
 	 */
-	if (!damaged) {
-		assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
-		assertEqualInt(1, archive_entry_mtime(ae));
-		/* Not the same as above: cpio doesn't store hi-res times. */
-		assert(0 == archive_entry_mtime_nsec(ae));
-		assert(0 == archive_entry_atime(ae));
-		assert(0 == archive_entry_ctime(ae));
-		assertEqualString("file2", archive_entry_pathname(ae));
-		assert((S_IFREG | 0755) == archive_entry_mode(ae));
-		assertEqualInt(4, archive_entry_size(ae));
-		assertEqualIntA(a, 4, archive_read_data(a, filedata, 10));
-		assert(0 == memcmp(filedata, "1234", 4));
-	}
+
+	/*
+	 * Read the third file back.
+	 * ARCHIVE_WARN here because the damaged entry was skipped.
+	 */
+	assertEqualIntA(a, ARCHIVE_WARN, archive_read_next_header(a, &ae));
+	assertEqualString("name", archive_entry_pathname(ae));
 
 	/*
 	 * Read the dir entry back.
 	 */
-	assertEqualIntA(a,
-	    damaged ? ARCHIVE_WARN : ARCHIVE_OK,
-	    archive_read_next_header(a, &ae));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
 	assertEqualInt(11, archive_entry_mtime(ae));
 	assert(0 == archive_entry_mtime_nsec(ae));
 	assert(0 == archive_entry_atime(ae));
@@ -181,23 +214,57 @@ test_format(int	(*set_format)(struct archive *))
 
 	/* Verify the end of the archive. */
 	assertEqualIntA(a, 1, archive_read_next_header(a, &ae));
-	assert(0 == archive_read_close(a));
-#if ARCHIVE_API_VERSION > 1
-	assert(0 == archive_read_finish(a));
-#else
-	archive_read_finish(a);
-#endif
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
 
 	free(buff);
 }
-#endif
+
+static void
+test_big_entries(int (*set_format)(struct archive *), int64_t size, int expected)
+{
+	struct archive_entry *ae;
+	struct archive *a;
+	size_t buffsize = 1000000;
+	size_t used;
+	char *buff;
+
+	buff = malloc(buffsize);
+
+	/* Create a new archive in memory. */
+	assert((a = archive_write_new()) != NULL);
+	assertA(0 == (*set_format)(a));
+	assertA(0 == archive_write_set_compression_none(a));
+	assertA(0 == archive_write_open_memory(a, buff, buffsize, &used));
+
+	assert((ae = archive_entry_new()) != NULL);
+	archive_entry_copy_pathname(ae, "file");
+	archive_entry_set_size(ae, size);
+	archive_entry_set_filetype(ae, AE_IFREG);
+	assertEqualInt(expected, archive_write_header(a, ae));
+	if (expected != ARCHIVE_OK)
+		assert(archive_error_string(a) != NULL);
+
+	archive_entry_free(ae);
+	archive_write_free(a);
+	free(buff);
+}
+
 
 DEFINE_TEST(test_write_format_cpio)
 {
-#if ARCHIVE_VERSION_STAMP >= 1009000
+	int64_t size_4g = ((int64_t)1) << 32;
+	int64_t size_8g = ((int64_t)1) << 33;
+
 	test_format(archive_write_set_format_cpio);
 	test_format(archive_write_set_format_cpio_newc);
-#else
-	skipping("cpio write support");
-#endif
+
+	test_big_entries(archive_write_set_format_cpio,
+	    size_8g - 1, ARCHIVE_OK);
+	test_big_entries(archive_write_set_format_cpio,
+	    size_8g, ARCHIVE_FAILED);
+	test_big_entries(archive_write_set_format_cpio_newc,
+	    size_4g - 1, ARCHIVE_OK);
+	test_big_entries(archive_write_set_format_cpio_newc,
+	    size_4g, ARCHIVE_FAILED);
 }
