@@ -28,8 +28,6 @@
 #include "bsdtar_platform.h"
 #include <stdio.h>
 
-#include "matching.h"
-
 #define	DEFAULT_BYTES_PER_BLOCK	(20*512)
 
 /*
@@ -46,16 +44,11 @@ struct bsdtar {
 	const char	 *create_format; /* -F format */
 	char		 *pending_chdir; /* -C dir */
 	const char	 *names_from_file; /* -T file */
-	int		  newer_ctime_filter; /* --newer/--newer-than */
-	time_t		  newer_ctime_sec; /* --newer/--newer-than */
-	long		  newer_ctime_nsec; /* --newer/--newer-than */
-	int		  newer_mtime_filter; /* --newer-mtime/--newer-mtime-than */
-	time_t		  newer_mtime_sec; /* --newer-mtime */
-	long		  newer_mtime_nsec; /* --newer-mtime-than */
 	int		  bytes_per_block; /* -b block_size */
 	int		  bytes_in_last_block; /* See -b handling. */
 	int		  verbose;   /* -v */
 	int		  extract_flags; /* Flags for extract operation */
+	int		  readdisk_flags; /* Flags for read disk operation */
 	int		  strip_components; /* Remove this many leading dirs */
 	int		  gid;  /* --gid */
 	const char	 *gname; /* --gname */
@@ -67,10 +60,8 @@ struct bsdtar {
 	const char	 *compress_program;
 	char		  option_absolute_paths; /* -P */
 	char		  option_chroot; /* --chroot */
-	char		  option_dont_traverse_mounts; /* --one-file-system */
 	char		  option_fast_read; /* --fast-read */
 	const char	 *option_options; /* --options */
-	char		  option_honor_nodump; /* --nodump */
 	char		  option_interactive; /* -w */
 	char		  option_no_owner; /* -o */
 	char		  option_no_subdirs; /* -n */
@@ -81,7 +72,6 @@ struct bsdtar {
 	char		  option_unlink_first; /* -U */
 	char		  option_warn_links; /* --check-links */
 	char		  day_first; /* show day before month in -tv output */
-	char		  enable_copyfile; /* For Mac OS */
 
 	/* Option parser state */
 	int		  getopt_state;
@@ -111,7 +101,8 @@ struct bsdtar {
 	struct name_cache	*gname_cache;	/* for write.c */
 	char			*buff;		/* for write.c */
 	size_t			 buff_size;	/* for write.c */
-	struct lafe_matching	*matching;	/* for matching.c */
+	int			 first_fs;	/* for write.c */
+	struct archive		*matching;	/* for matching.c */
 	struct security		*security;	/* for read.c */
 	struct name_cache	*uname_cache;	/* for write.c */
 	struct siginfo_data	*siginfo;	/* for siginfo.c */
