@@ -38,11 +38,13 @@ __FBSDID("$FreeBSD: src/usr.bin/tar/subst.c,v 1.4 2008/06/15 10:08:16 kientzle E
 #define	REG_BASIC 0
 #endif
 
+#include "err.h"
+
 struct subst_rule {
 	struct subst_rule *next;
 	regex_t re;
 	char *result;
-	unsigned int global:1, print:1, symlink:1;
+	unsigned int global:1, print:1, regular:1, symlink:1, hardlink:1;
 };
 
 struct substitution {
@@ -56,7 +58,7 @@ init_substitution(struct bsdtar *bsdtar)
 
 	bsdtar->substitution = subst = malloc(sizeof(*subst));
 	if (subst == NULL)
-		bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+		lafe_errc(1, errno, "Out of memory");
 	subst->first_rule = subst->last_rule = NULL;
 }
 
@@ -76,7 +78,7 @@ add_substitution(struct bsdtar *bsdtar, const char *rule_text)
 
 	rule = malloc(sizeof(*rule));
 	if (rule == NULL)
-		bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+		lafe_errc(1, errno, "Out of memory");
 	rule->next = NULL;
 
 	if (subst->last_rule == NULL)
@@ -86,38 +88,41 @@ add_substitution(struct bsdtar *bsdtar, const char *rule_text)
 	subst->last_rule = rule;
 
 	if (*rule_text == '\0')
-		bsdtar_errc(bsdtar, 1, 0, "Empty replacement string");
+		lafe_errc(1, 0, "Empty replacement string");
 	end_pattern = strchr(rule_text + 1, *rule_text);
 	if (end_pattern == NULL)
-		bsdtar_errc(bsdtar, 1, 0, "Invalid replacement string");
+		lafe_errc(1, 0, "Invalid replacement string");
 
 	pattern = malloc(end_pattern - rule_text);
 	if (pattern == NULL)
-		bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+		lafe_errc(1, errno, "Out of memory");
 	memcpy(pattern, rule_text + 1, end_pattern - rule_text - 1);
 	pattern[end_pattern - rule_text - 1] = '\0';
 
 	if ((r = regcomp(&rule->re, pattern, REG_BASIC)) != 0) {
 		char buf[80];
 		regerror(r, &rule->re, buf, sizeof(buf));
-		bsdtar_errc(bsdtar, 1, 0, "Invalid regular expression: %s", buf);
+		lafe_errc(1, 0, "Invalid regular expression: %s", buf);
 	}
 	free(pattern);
 
 	start_subst = end_pattern + 1;
 	end_pattern = strchr(start_subst, *rule_text);
 	if (end_pattern == NULL)
-		bsdtar_errc(bsdtar, 1, 0, "Invalid replacement string");
+		lafe_errc(1, 0, "Invalid replacement string");
 
 	rule->result = malloc(end_pattern - start_subst + 1);
 	if (rule->result == NULL)
-		bsdtar_errc(bsdtar, 1, errno, "Out of memory");
+		lafe_errc(1, errno, "Out of memory");
 	memcpy(rule->result, start_subst, end_pattern - start_subst);
 	rule->result[end_pattern - start_subst] = '\0';
 
-	rule->global = 0;
-	rule->print = 0;
-	rule->symlink = 0;
+	/* Defaults */
+	rule->global = 0; /* Don't do multiple replacements. */
+	rule->print = 0; /* Don't print. */
+	rule->regular = 1; /* Rewrite regular filenames. */
+	rule->symlink = 1; /* Rewrite symlink targets. */
+	rule->hardlink = 1; /* Rewrite hardlink targets. */
 
 	while (*++end_pattern) {
 		switch (*end_pattern) {
@@ -125,22 +130,36 @@ add_substitution(struct bsdtar *bsdtar, const char *rule_text)
 		case 'G':
 			rule->global = 1;
 			break;
+		case 'h':
+			rule->hardlink = 1;
+			break;
+		case 'H':
+			rule->hardlink = 0;
+			break;
 		case 'p':
 		case 'P':
 			rule->print = 1;
 			break;
+		case 'r':
+			rule->regular = 1;
+			break;
+		case 'R':
+			rule->regular = 0;
+			break;
 		case 's':
-		case 'S':
 			rule->symlink = 1;
 			break;
+		case 'S':
+			rule->symlink = 0;
+			break;
 		default:
-			bsdtar_errc(bsdtar, 1, 0, "Invalid replacement flag %c", *end_pattern);
+			lafe_errc(1, 0, "Invalid replacement flag %c", *end_pattern);
 		}
 	}
 }
 
 static void
-realloc_strncat(struct bsdtar *bsdtar, char **str, const char *append, size_t len)
+realloc_strncat(char **str, const char *append, size_t len)
 {
 	char *new_str;
 	size_t old_len;
@@ -152,8 +171,9 @@ realloc_strncat(struct bsdtar *bsdtar, char **str, const char *append, size_t le
 
 	new_str = malloc(old_len + len + 1);
 	if (new_str == NULL)
-		bsdtar_errc(bsdtar, 1, errno, "Out of memory");
-	memcpy(new_str, *str, old_len);
+		lafe_errc(1, errno, "Out of memory");
+	if (*str != NULL)
+		memcpy(new_str, *str, old_len);
 	memcpy(new_str + old_len, append, len);
 	new_str[old_len + len] = '\0';
 	free(*str);
@@ -161,7 +181,7 @@ realloc_strncat(struct bsdtar *bsdtar, char **str, const char *append, size_t le
 }
 
 static void
-realloc_strcat(struct bsdtar *bsdtar, char **str, const char *append)
+realloc_strcat(char **str, const char *append)
 {
 	char *new_str;
 	size_t old_len;
@@ -173,15 +193,17 @@ realloc_strcat(struct bsdtar *bsdtar, char **str, const char *append)
 
 	new_str = malloc(old_len + strlen(append) + 1);
 	if (new_str == NULL)
-		bsdtar_errc(bsdtar, 1, errno, "Out of memory");
-	memcpy(new_str, *str, old_len);
+		lafe_errc(1, errno, "Out of memory");
+	if (*str != NULL)
+		memcpy(new_str, *str, old_len);
 	strcpy(new_str + old_len, append);
 	free(*str);
 	*str = new_str;
 }
 
 int
-apply_substitution(struct bsdtar *bsdtar, const char *name, char **result, int symlink_only)
+apply_substitution(struct bsdtar *bsdtar, const char *name, char **result,
+    int symlink_target, int hardlink_target)
 {
 	const char *path = name;
 	regmatch_t matches[10];
@@ -199,19 +221,30 @@ apply_substitution(struct bsdtar *bsdtar, const char *name, char **result, int s
 	print_match = 0;
 
 	for (rule = subst->first_rule; rule != NULL; rule = rule->next) {
-		if (symlink_only && !rule->symlink)
-			continue;
+		if (symlink_target) {
+			if (!rule->symlink)
+				continue;
+		} else if (hardlink_target) {
+			if (!rule->hardlink)
+				continue;
+		} else { /* Regular filename. */
+			if (!rule->regular)
+				continue;
+		}
+
 		if (regexec(&rule->re, name, 10, matches, 0))
 			continue;
 
 		got_match = 1;
 		print_match |= rule->print;
-		realloc_strncat(bsdtar, result, name, matches[0].rm_so);
+		realloc_strncat(result, name, matches[0].rm_so);
 
 		for (i = 0, j = 0; rule->result[i] != '\0'; ++i) {
 			if (rule->result[i] == '~') {
-				realloc_strncat(bsdtar, result, rule->result + j, i - j);
-				realloc_strncat(bsdtar, result, name, matches[0].rm_eo);
+				realloc_strncat(result, rule->result + j, i - j);
+				realloc_strncat(result,
+				    name + matches[0].rm_so,
+				    matches[0].rm_eo - matches[0].rm_so);
 				j = i + 1;
 				continue;
 			}
@@ -223,7 +256,7 @@ apply_substitution(struct bsdtar *bsdtar, const char *name, char **result, int s
 			switch (c) {
 			case '~':
 			case '\\':
-				realloc_strncat(bsdtar, result, rule->result + j, i - j - 1);
+				realloc_strncat(result, rule->result + j, i - j - 1);
 				j = i;
 				break;
 			case '1':
@@ -235,13 +268,13 @@ apply_substitution(struct bsdtar *bsdtar, const char *name, char **result, int s
 			case '7':
 			case '8':
 			case '9':
-				realloc_strncat(bsdtar, result, rule->result + j, i - j - 1);
+				realloc_strncat(result, rule->result + j, i - j - 1);
 				if ((size_t)(c - '0') > (size_t)(rule->re.re_nsub)) {
 					free(*result);
 					*result = NULL;
 					return -1;
 				}
-				realloc_strncat(bsdtar, result, name + matches[c - '0'].rm_so, matches[c - '0'].rm_eo - matches[c - '0'].rm_so);
+				realloc_strncat(result, name + matches[c - '0'].rm_so, matches[c - '0'].rm_eo - matches[c - '0'].rm_so);
 				j = i + 1;
 				break;
 			default:
@@ -251,7 +284,7 @@ apply_substitution(struct bsdtar *bsdtar, const char *name, char **result, int s
 
 		}
 
-		realloc_strcat(bsdtar, result, rule->result + j);
+		realloc_strcat(result, rule->result + j);
 
 		name += matches[0].rm_eo;
 
@@ -260,7 +293,7 @@ apply_substitution(struct bsdtar *bsdtar, const char *name, char **result, int s
 	}
 
 	if (got_match)
-		realloc_strcat(bsdtar, result, name);
+		realloc_strcat(result, name);
 
 	if (print_match)
 		fprintf(stderr, "%s >> %s\n", path, *result);
