@@ -2,7 +2,7 @@ DEFINE_TEST(test_0)
 DEFINE_TEST(test_basic)
 DEFINE_TEST(test_copy)
 DEFINE_TEST(test_empty_mtree)
-DEFINE_TEST(test_getdate)
+DEFINE_TEST(test_format_newc)
 DEFINE_TEST(test_help)
 DEFINE_TEST(test_option_C_upper)
 DEFINE_TEST(test_option_H_upper)
@@ -18,6 +18,7 @@ DEFINE_TEST(test_option_k)
 DEFINE_TEST(test_option_keep_newer_files)
 DEFINE_TEST(test_option_n)
 DEFINE_TEST(test_option_newer_than)
+DEFINE_TEST(test_option_nodump)
 DEFINE_TEST(test_option_q)
 DEFINE_TEST(test_option_r)
 DEFINE_TEST(test_option_s)
